#include <cstdio>
#include "shallowwater.h"

using namespace std;

int main(int argc, char **argv)
{
   if(argc<2){
      fprintf(stderr, "usage: compute_water <configuration_filename> [end time=10] [frame rate=24]\n");
      return 1;
   }

   ShallowWater water;
   water.read_config(argv[1]);

   float end_time=10, frame_rate=24;
   if(argc>=3) sscanf(argv[2], "%f", &end_time);
   if(argc>=4) sscanf(argv[3], "%f", &frame_rate);

   // save the initial conditions
   water.write("water%04d", 0);

   int steps=(int)(end_time*frame_rate);
   for(int i=1; i<=steps; ++i){
      printf("advancing to frame %d (of %d)\n", i, steps);
      // advance a frame
      float t=0, endt=1./frame_rate;
      bool done=false;
      while(!done){
	 float dt=water.maxdt();
	 if(t+dt>=endt){
	    done=true;
	    dt=endt-t;
	 }else if(t+1.5*dt>=endt)
	    dt=0.5*(endt-t);
	 water.advance(dt);
	 t+=dt;
      }
      water.write("water%04d", i);
   }

   return 0;
}

