#ifndef SHALLOWWATER_H
#define SHALLOWWATER_H

#include "array2f.h"

struct ShallowWater{
   int nx, nz;
   Array2f h, u, w;
   Array2f newh, newu, neww;
   Array2f solidh, solidu, solidw;     // level sets of solids inside domain (negative means solid, positive means water)
   float lx, lz;      // dimensions in metres
   float dx, overdx;  // grid cell spacing in metres (and reciprocal in 1/m)
   float density;     // fluid density in kg/m^3
   float gravity;     // acceleration due to gravity in m/s^2

   ShallowWater()
      : dx(0), density(1000), gravity(9.8)
   {}

   void init(float lx, int cell_nx, int cell_nz);
   void read_config(const char *filename);
   void write(const char *filename_format, ...);
   float maxdt(void);
   void advance(float dt);

   private:
   void fill_in_old_height(void);
   void advect_height(float dt);
   void fix_height(float dt);
   void advect_velocity(float dt);
   void apply_pressure(float dt);
   void switch_old_and_new(void);
};

#endif
