#ifndef ARRAY2F
#define ARRAY2F

#include <cstdio>
#include <cmath>
#include <cstring>

struct Array2f{
   int nx, ny;
   int size;
   float *data;

   Array2f()
      :nx(0), ny(0), size(0), data(0)
   {}

   Array2f(int nx_, int ny_)
      :nx(0), ny(0), size(0), data(0)
   { init(nx_, ny_); }

   void init(int nx_, int ny_)
   {
      if(nx!=nx_ || ny!=ny_){
         delete_memory();
         nx=nx_;
         ny=ny_;
         size=nx*ny;
         data=new float[size];
      }
      zero();
   }

   ~Array2f()
   { delete_memory(); }

   void delete_memory()
   {
      delete[] data; data=0;
      nx=ny=size=0;
   }

   const float &operator() (int i, int j) const
   { return data[i+nx*j]; }

   float &operator() (int i, int j)
   { return data[i+nx*j]; }

   float bilerp(int i, int j, float fx, float fy)
   { return (1-fx)*((1-fy)*(*this)(i,j)+fy*(*this)(i,j+1))+fx*((1-fy)*(*this)(i+1,j)+fy*(*this)(i+1,j+1)); }

   void copy_to(Array2f &a) const
   { std::memcpy(a.data, data, size*sizeof(float)); }

   float infnorm() const
   { 
      float r=0;
      for(int i=0; i<size; ++i)
         if(!(std::fabs(data[i])<=r)) r=std::fabs(data[i]);
      return r;
   }

   void zero()
   { std::memset(data, 0, size*sizeof(float)); }

   double dot(const Array2f &a) const
   {
      double r=0;
      for(int i=0; i<size; ++i)
         r+=data[i]*a.data[i];
      return r;
   }

   void increment(double scale, const Array2f &a)
   { for(int i=0; i<size; ++i) data[i]+=scale*a.data[i]; }

   void scale_and_increment(double scale, const Array2f &a)
   { for(int i=0; i<size; ++i) data[i]=scale*data[i]+a.data[i]; }

   void write_matlab(FILE *fp, const char *variable_name)
   {
      fprintf(fp, "%s=[", variable_name);
      for(int i=0; i<nx; ++i){
	 for(int j=0; j<ny; ++j)
	    fprintf(fp, "%lg ", (double)(*this)(i,j));
	 fprintf(fp, ";\n");
      }
      fprintf(fp,"];\n");
   }
};

#endif
