#include <cstdio>
#include <iostream>
#include <fstream>
#include "array2f.h"
#include "gluvi.h"
#include "vec3.h"

using namespace std;

const char *filename_format;
char frame_number[100]="frame 0";
unsigned int frame=0;
float lx;
Array2f h;

void read_frame(int newframe)
{
   char filename[256];
   sprintf(filename, filename_format, newframe);

   FILE *in=fopen(filename, "rt");
   if(!in) return;

   int nx, ny;
   int i, j;
   fscanf(in, "%d %d", &nx, &ny);
   h.init(nx, ny);

   fscanf(in, "%f", &lx);

   // skip over useless stuff
   float junk;
   for(i=0; i<(2*nx+1)*(2*ny+1); ++i)
      fscanf(in, "%f", &junk);

   // read the heights
   for(j=0; j<ny; ++j) for(i=0; i<nx; ++i){
      fscanf(in, "%f", &h(i,j));
   }

   fclose(in);
   frame=newframe;
   sprintf(frame_number, "frame %d", frame);
}

void set_view(Gluvi::Target3D &cam)
{
   cam.dist=3*lx;
   cam.target[0]=lx/2;
   cam.target[2]=lx/2;
}

void display(void)
{
   int i, j;
   glDisable(GL_LIGHTING);

   // draw the water
   glColor3f(0.8, 0.8, 1);
   glBegin(GL_LINES);
   for(j=0; j<h.ny; ++j) for(i=1; i<h.nx; ++i){
      glVertex3f(lx*(i+0.5)/h.nx, h(i,j), lx*(j+0.5)/h.ny);
      glVertex3f(lx*(i-0.5)/h.nx, h(i-1,j), lx*(j+0.5)/h.ny);
   }
   for(j=1; j<h.ny; ++j) for(i=0; i<h.nx; ++i){
      glVertex3f(lx*(i+0.5)/h.nx, h(i,j), lx*(j+0.5)/h.ny);
      glVertex3f(lx*(i+0.5)/h.nx, h(i,j-1), lx*(j-0.5)/h.ny);
   }
   glEnd();

   // draw the tank
   glColor3f(0.5, 0.5, 0.7);
   glBegin(GL_QUADS);
   glVertex3f(0, 0, 0);
   glVertex3f(lx, 0, 0);
   glVertex3f(lx, 0, h.ny*lx/h.nx);
   glVertex3f(0, 0, h.ny*lx/h.nx);
   glEnd();
   glBegin(GL_LINES);
   glVertex3f(0, 0, 0);
   glVertex3f(0, lx, 0);
   glVertex3f(lx, 0, 0);
   glVertex3f(lx, lx, 0);
   glVertex3f(lx, 0, h.ny*lx/h.nx);
   glVertex3f(lx, lx, h.ny*lx/h.nx);
   glVertex3f(0, 0, h.ny*lx/h.nx);
   glVertex3f(0, lx, h.ny*lx/h.nx);
   glEnd();
}

struct CameraButton : public Gluvi::Button {
   CameraButton(const char *t) : Gluvi::Button(t) {}
   void action()
   {
      ofstream camera_output("water_camera.rib");
      Gluvi::camera->export_rib(camera_output);
   }
};

struct ScreenShotButton : public Gluvi::Button{
   const char *filename_format;
   ScreenShotButton(const char *label, const char *filename_format_) : Gluvi::Button(label), filename_format(filename_format_) {}
   void action()
   { Gluvi::ppm_screenshot(filename_format, frame); }
};

void special_key_handler(int key, int mousex, int mousey)
{
   switch(key){
      case GLUT_KEY_LEFT:
	 read_frame(frame-1);
         glutPostRedisplay();
         break;
      case GLUT_KEY_RIGHT:
	 read_frame(frame+1);
         glutPostRedisplay();
         break;
      default:
         ;
   }
}

int main(int argc, char **argv)
{
   Gluvi::init("opengl_water", &argc, argv);

   if(argc!=2){
      cerr<<"Expecting one argument: animation filename format"<<endl;
      return 1;
   }
   filename_format=argv[1];

   frame=0;
   read_frame(0);

   glutSpecialFunc(special_key_handler);

   Gluvi::Target3D cam;
   set_view(cam);
   Gluvi::camera=&cam;

   Gluvi::userDisplayFunc=display;

   Gluvi::StaticText frametext(frame_number);
   Gluvi::root.list.push_back(&frametext);

   Gluvi::StaticText keytext("use left/right arrows for frame control");
   Gluvi::root.list.push_back(&keytext);

   CameraButton cambutton("Export camera to RIB");
   Gluvi::root.list.push_back(&cambutton);

   char ppmfileformat[strlen(argv[1])+5];
   sprintf(ppmfileformat, "%s.ppm", argv[1]);
   ScreenShotButton screenshot("Screenshot", ppmfileformat);
   Gluvi::root.list.push_back(&screenshot);

   Gluvi::run();
   return 0;
}

